/*
Copyright © 2022 SUSE LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1

import (
	"syscall"
)

type SyscallInterface interface {
	Chroot(string) error
	Chdir(string) error
	Mount(string, string, string, uintptr, string) error
	Syscall(uintptr, uintptr, uintptr, uintptr) (uintptr, uintptr, syscall.Errno)
}

type RealSyscall struct{}

func (r *RealSyscall) Chroot(path string) error {
	return syscall.Chroot(path)
}

func (r *RealSyscall) Chdir(path string) error {
	return syscall.Chdir(path)
}
func (r *RealSyscall) Mount(source string, target string, fstype string, flags uintptr, data string) error {
	return syscall.Mount(source, target, fstype, flags, data)
}

func (r *RealSyscall) Syscall(trap, a1, a2, a3 uintptr) (r1, r2 uintptr, err syscall.Errno) {
	return syscall.Syscall(trap, a1, a2, a3)
}
